"""
Default settings for cratis project
"""
import inject
import os

from configurations import Configuration


class CratisConfig(Configuration):
    """
    Base class for Configurations that will be used with cratis.

    Class contains hooks pre_setup and post_setup that will load
    and configure Features defined in FEATURES settings variable.

    Also class defines ROOT_URLCONF settings variable, that use cratis.urls
    to load all urls collected from your Features. Usually you wouldn't change this
    when working with cratis.

    BASE_DIR just points to project directory, so you can specify paths relatively, ex:

    STATIC_ROOT = os.path.join(CratisConfig.BASE_DIR, 'some dir')

    Order of Feature load is following:

     - Call set_settings() on each Feature, providing class reference, so Feature can save it for future use.
     - Call do_load() on each Feature, passing previously loaded feature list as an argument
     - .. now django-configurations executes own internal things..
     - Django is fully configured, we run on_after_load on each Feature, so Features can do latest adjustments
     - And finally call on_startup() on each feature, features shouldn't modify anything on this step.

    """

    BASE_DIR = os.environ.get('CRATIS_APP_PATH', '.')
    ROOT_URLCONF = 'cratis.urls'

    FEATURES = ()

    @classmethod
    def pre_setup(cls):
        super(CratisConfig, cls).pre_setup()

        loaded_features = []
        for feature in cls.FEATURES:
            feature.set_settings(cls)
            feature.on_load(loaded_features)
            loaded_features.append(type(feature))


    @classmethod
    def post_setup(cls):
        super(CratisConfig, cls).post_setup()

        def configure_feature_services(binder):
            for feature in cls.FEATURES:
                feature.configure_services(binder)

        inject.configure(configure_feature_services)

        for feature in cls.FEATURES:
            feature.on_after_load()

        for feature in cls.FEATURES:
            feature.on_startup()

    @classmethod
    def override_feature(cls, feature):
        cls.FEATURES = tuple([x if not isinstance(x, type(feature)) else feature for x in cls.FEATURES])

    @classmethod
    def add_feature(cls, feature):
        cls.FEATURES += (feature,)






import sys

class FeatureDependencyFailure(Exception):
    pass


class BaseFeature(object):
    """
    Basement for all the features.
    """
    settings = None

    def set_settings(self, cls):
        """
        Initialized just before configuration process start
        """
        self.settings = cls

    def get_required(self):
        """
        Returns list of classes of features that must be loaded before this feature
        can be loaded
        """
        return ()

    def on_load(self, loaded_features):
        """
        Loads feature. Used by settings class.
        By default, checks requirements and executes configure_settings() method.

        As an argument accepts list of features loaded before current one.
        """

        for requirement in self.get_required():
            if not requirement in loaded_features:
                message = '\n Feature %s depends on %s, that is not loaded yet. Order is also important!\n' % (
                    str(type(self)),
                    str(requirement),
                )
                raise FeatureDependencyFailure(message)

        self.configure_settings()


    def on_after_load(self):
        """
        Called when setting is configured
        """
        pass


    def on_startup(self):
        """
        Last chance to do something after settings are configured, called even later than on_after_load.
        """
        pass


    def configure_settings(self):
        """
        API method.
        Meant to be overridden by subsequent Features.

        Called inside on_load callback.
        """
        pass

    def configure_urls(self, urls):
        """
        API method.
        Meant to be overridden by subsequent Features.

        Called when django imports cratis.url from cratis.urls module.

        As a parameter accepts urlpatterns variable from cratis.urls
        """

    def configure_services(self, binder):
        """
        API method.
        Meant to be overridden by subsequent Features.

        Called when services injector configuration happens

        As a parameter accepts binder instance.

        See python inject for details.
        """


class Feature(BaseFeature):
    """
    Feature add some concreate functionality to the BaseFeature class.
    """

    def append_apps(self, apps):
        """
        Use this in configure_settings, to append new INSTALLED_APPS.
        """

        if isinstance(apps, basestring):
            apps = (apps,)

        for app in apps:
            if app not in self.settings.INSTALLED_APPS:
                self.settings.INSTALLED_APPS += (app,)

    def append_middleware(self, classes):
        """
        Use this in configure_settings, to append new middleware classes.
        """

        if isinstance(classes, basestring):
            classes = (classes,)

        for classname in classes:
            if classname not in self.settings.MIDDLEWARE_CLASSES:
                self.settings.MIDDLEWARE_CLASSES += (classname,)

    def append_template_processor(self, processors):
        """
        Use this in configure_settings, to append new template processors.
        """
        if isinstance(processors, basestring):
            processors = (processors,)

        for classname in processors:
            if classname not in self.settings.TEMPLATE_CONTEXT_PROCESSORS:
                self.settings.TEMPLATE_CONTEXT_PROCESSORS += (classname,)



import os
import sys
import yaml
import string
import random


def read_yaml_file(_file):
    with open(_file) as f:
        return yaml.load(f)


def _find_project_env(path):
    """
    Finds cratis.yml for cratis command.

    Algorithm is following:

    * Check current directory
    * If no cratis.yml, check parent and reapeat until found

        * Nothing found, return None
        * If found, check if parent also contains cratis.yml

            * If parent contains, then return parent's cratis.yml path
            * If no, then return current dir cratis.yml

    """
    env_file_name = '%s/cratis.yml' % path
    env_parent_file_name = '%s/cratis.yml' % os.path.dirname(path)

    if os.path.exists(env_file_name):
        if os.path.exists(env_parent_file_name):
            return env_parent_file_name
        return env_file_name
    elif path != '/':
        return _find_project_env(os.path.dirname(path))
    else:
        return None


class NotProperlyConfigured(Exception):
    pass


class NoVariableDefined(NotProperlyConfigured):
    def __init__(self, var_name, *args, **kwargs):
        super(NoVariableDefined, self).__init__(*args, **kwargs)
        self.var_name = var_name


class NoConfigFile(NotProperlyConfigured):
    pass


class NoCratisApp(NotProperlyConfigured):
    pass


def load_env():

    """
    Loads environment variables defined in cratis.yml file.

    load_env() finds cratis.yml file in current directory, then in it's parent and
    so on until it finds, or root dir reached ("/").
    If no cratis.yml found, then error shown.

    Some variables are required for cratis to work::

    - CRATIS_APP_PATH - Path where cratis application is located. (default ".")
    - DJANGO_SETTINGS_MODULE - Where to find settings module (Default "settings", means settings.py file in CRATIS_APP_PATH)
    - DJANGO_CONFIGURATION - One of Environments defined in settings.py (default "Dev")

    *Dev mode*

    In development mode all valuesare meant to be used with default values, and cratis.yml
    is just a marker of where your project is located. Very useful when you execute cratis
    command from subdirectory of your project (yes, you can do this!).

    Structure in this case will be following::

        - myproject/
          - .git
          - settings.py
          - cratis.yml

    *Production mode*

    In production mode, cratis.yml plays important role of providing environment information.

    Structure in this case will be following::

        - myproject/
          - src/
            - .git
            - settings.py
            - cratis.yml
          - cratis.yml

    Note: cratis.yml that is located one level up will always get priority.

    cratis.yml file contents::

        CRATIS_APP_PATH': myproject,
        DJANGO_CONFIGURATION': Prod,
        DJANGO_SETTINGS_MODULE: settings

        SOME_OTHER_ENV_VARIABLE: xxx
        ...

    Django-Configurations highly relays on ENV variables, so you can easily redefine some settings
    through Env variables.

    """
    _file = _find_project_env(os.getcwd())
    if not _file:
        raise NotProperlyConfigured('Can not find cratis.yml in current directory or any of parents.')
    else:
        for key, val in read_yaml_file(_file).iteritems():
            val = str(val)
            if not key in os.environ:
                os.environ[key] = val

    if not "CRATIS_APP_PATH" in os.environ:
        raise NoVariableDefined("CRATIS_APP_PATH", 'Found cratis.yml, but it do not contain CRATIS_APP_PATH. Corrupted file?')

    if not "DJANGO_CONFIGURATION" in os.environ:
        raise NoVariableDefined("DJANGO_CONFIGURATION", 'Found cratis.yml, but it do not contain DJANGO_CONFIGURATION. Corrupted file?')

    if not "DJANGO_SETTINGS_MODULE" in os.environ:
        raise NoVariableDefined("DJANGO_SETTINGS_MODULE", 'Found cratis.yml, but it do not contain DJANGO_SETTINGS_MODULE. Corrupted file?')

    if not os.path.exists(os.environ["CRATIS_APP_PATH"]):
        raise NoCratisApp('Path specified in CRATIS_APP_PATH do not exist: %s' % os.environ["CRATIS_APP_PATH"])

    sys.path += (os.environ["CRATIS_APP_PATH"], )


def cratis_cmd():
    """
    Command that executes django ./manage.py task + loads environment variables from cratis.yml

    Command also can be executed from sub-folders of project.
    """

    try:
        load_env()

        from configurations.management import execute_from_command_line

        execute_from_command_line(sys.argv)

    except NotProperlyConfigured as e:
        print('\n\tError: %s\n' % e.message)





class AlreadyExistsException(Exception):
    pass


def cratis_init_cmd():
    """
    Command creates new cratis project in current directory.
    Two files are generated:

    * Empty settings file
    * cratis.yml configured to serve project from current directory.

    Usage::

      cratis-init
      cratis runserver

    """
    path = os.path.join(os.getcwd(), 'cratis.yml')
    path_settings = os.path.join(os.getcwd(), 'settings.py')

    if os.path.exists(path):
        raise AlreadyExistsException('File cratis.yml already exist in current directory.')
    else:
        with open(path, 'w+') as f:
            yaml.dump({
                'CRATIS_APP_PATH': '.',
                'DJANGO_CONFIGURATION': 'Dev',
                'DJANGO_SETTINGS_MODULE': 'settings'
            }, f, default_flow_style=False)

    if os.path.exists(path_settings):
        raise AlreadyExistsException('File settings.py already exist in current directory.')
    else:
        with open(path_settings, 'w+') as f:
            f.write('''# coding=utf-8
from cratis.settings import CratisConfig


class Dev(CratisConfig):
    DEBUG = True
    SECRET_KEY = '%s'

    FEATURES = (
        # your features here
    )


class Test(Dev):
    pass


class Prod(Dev):
    DEBUG = False

    FEATURES = Dev.FEATURES + (
        # your features here
    )
''' %  ''.join([random.choice(string.ascii_letters + string.digits) for n in xrange(30)]))


import os
from setuptools import setup, find_packages
# from tests import PyTest

setup(
    name='django-cratis',
    version='0.5.0',
    packages=find_packages(),

    url='',
    license='Simplified BSD License',
    author='Alex Rudakov',
    author_email='ribozz@gmail.com',
    description='Django-cratis is a way to group together django applications, so they form reusable features.',
    long_description='',
    install_requires=[
        'django>=1.4',
        'django-configurations',
        'dj-database-url',
        'PyYaml',
        'inject'
    ],

    entry_points={
        'console_scripts': [
            'cratis = cratis.cli:cratis_cmd',
            'cratis-init = cratis.cli:cratis_init_cmd',
        ],
    },
)




from configurations import importer
from cratis.cli import load_env

load_env()

importer.install()

import os
print os.environ['DJANGO_SETTINGS_MODULE']
try:
    from django.core.wsgi import get_wsgi_application
except ImportError:  # pragma: no cover
    from django.core.handlers.wsgi import WSGIHandler

    def get_wsgi_application():  # noqa
        return WSGIHandler()

# this is just for the crazy ones
application = get_wsgi_application()
